const preferences = process.platform === 'darwin'
  ? require('bindings')('cf-prefs.node')
  : null;

module.exports = {
  getPreferenceValue: preferences ? preferences.getValue : () => undefined,
  isPreferenceForced: preferences ? preferences.isPreferenceForced : () => false,
};
